// --- bestaande includes en defines blijven hetzelfde ---
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <Adafruit_NeoPixel.h>

#define TRIG_PIN 9
#define ECHO_PIN 10
#define LED_PIN 6  
#define NUM_LEDS 16  
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800);

bool melodyPlayed = false;

#define melodyPin 7

LiquidCrystal_I2C lcd(0x27, 16, 2);

unsigned long noDetectionSince = 0;
bool wasDetecting = false;

unsigned long previousMillis = 0;
unsigned long elapsedTotalTime = 0;   // Totale tijd
unsigned long elapsedSessionTime = 0; // Tijd sinds laatste reset
bool counting = false;

int currentPixel = 0;
unsigned long lastColorUpdate = 0;
int colorIndex = 0;
const int maxColors = 2;
uint32_t colors[maxColors];

int melody[] = { 622, 587, 554 };
int noteDurations[] = { 150, 250, 350 };
const int melodyLength = 3;

int currentNote = 0;
unsigned long lastNoteTime = 0;
bool playingMelody = false;

void setup() {
  pinMode(TRIG_PIN, OUTPUT);
  pinMode(ECHO_PIN, INPUT);
  pinMode(melodyPin, OUTPUT);

  strip.begin();
  strip.show();

  Serial.begin(9600);

  lcd.init();
  lcd.backlight();

  lcd.setCursor(0, 0);
  lcd.print("Afstand:");

  for (int i = 0; i < NUM_LEDS; i++) {
    strip.setPixelColor(i, strip.Color(0, 0, 255)); // Blauw
  }
}

void loop() {
  updateLedProgress();

  float distance = measureDistance();
  lcd.setCursor(8, 0);

  if (distance < 70) {
    if (!counting) {
      previousMillis = millis();
    }
    counting = true;
  } else {
    counting = false;
  }

  if (counting) {
    updateTimers();
    updateColorsBasedOnTimer();   // hier toegevoegd
  }

  displayTimers();

  if (playingMelody) {
    playMelody();
  }
}

float measureDistance() {
  digitalWrite(TRIG_PIN, LOW);
  delayMicroseconds(2);
  digitalWrite(TRIG_PIN, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIG_PIN, LOW);

  long duration = pulseIn(ECHO_PIN, HIGH);
  return (duration * 0.0343) / 2;
}

void updateTimers() {
  unsigned long currentMillis = millis();
  unsigned long deltaTime = (currentMillis - previousMillis);
  elapsedTotalTime += deltaTime;
  elapsedSessionTime += deltaTime;
  previousMillis = currentMillis;
}

void displayTimers() {
  unsigned long totalSeconds = elapsedTotalTime / 1000;
  unsigned long totalHours = totalSeconds / 3600;
  unsigned long totalMinutes = (totalSeconds % 3600) / 60;
  totalSeconds = totalSeconds % 60;

  unsigned long sessionSeconds = elapsedSessionTime / 1000;
  unsigned long sessionHours = sessionSeconds / 3600;
  unsigned long sessionMinutes = (sessionSeconds % 3600) / 60;
  sessionSeconds = sessionSeconds % 60;

  lcd.setCursor(0, 0);
  lcd.print("TOT: ");
  lcd.print(totalHours);
  lcd.print(":");
  if (totalMinutes < 10) lcd.print("0");
  lcd.print(totalMinutes);
  lcd.print(":");
  if (totalSeconds < 10) lcd.print("0");
  lcd.print(totalSeconds);

  lcd.setCursor(0, 1);
  lcd.print("TIJD:");
  lcd.print(sessionHours);
  lcd.print(":");
  if (sessionMinutes < 10) lcd.print("0");
  lcd.print(sessionMinutes);
  lcd.print(":");
  if (sessionSeconds < 10) lcd.print("0");
  lcd.print(sessionSeconds);
}

void updateColorsBasedOnTimer() {
  static int lastPhase = -1;
  int currentPhase = 0;

  if (elapsedSessionTime < 30000) currentPhase = 0;
  else if (elapsedSessionTime < 60000) currentPhase = 1;
  else if (elapsedSessionTime < 90000) currentPhase = 2;
  else currentPhase = 3;

  if (currentPhase != lastPhase) {
    playingMelody = true;
    currentNote = 0;
    lastNoteTime = millis();
    lastPhase = currentPhase;
  }

  switch (currentPhase) {
    case 0:
      colors[0] = strip.Color(0, 255, 0);
      colors[1] = strip.Color(255, 255, 0);
      break;
    case 1:
      colors[0] = strip.Color(255, 255, 0);
      colors[1] = strip.Color(255, 0, 0);
      break;
    case 2:
      colors[0] = strip.Color(255, 0, 0);
      colors[1] = strip.Color(128, 0, 128);
      break;
    case 3:
      colors[0] = strip.Color(255, 255, 255);
      colors[1] = strip.Color(0, 255, 255);
      break;
  }
}

void updateColorWipe(unsigned long interval) {
  unsigned long currentMillis = millis();

  if (currentMillis - lastColorUpdate >= interval) {
    lastColorUpdate = currentMillis;

    strip.setPixelColor(currentPixel, colors[colorIndex]);
    strip.show();
    currentPixel++;

    if (currentPixel >= NUM_LEDS) {
      currentPixel = 0;
      colorIndex++;
      if (colorIndex >= maxColors) {
        colorIndex = 0;
      }
    }
  }
}

void playMelody() {
  unsigned long currentMillis = millis();

  if (currentNote < melodyLength && currentMillis - lastNoteTime >= noteDurations[currentNote]) {
    tone(melodyPin, melody[currentNote]);
    lastNoteTime = currentMillis;
    currentNote++;
  } else if (currentNote >= melodyLength) {
    noTone(melodyPin);
    playingMelody = false;
    currentNote = 0;
  }
}

void updateLedProgress() {
  float seconds = elapsedSessionTime / 1000.0;
  unsigned long currentMillis = millis();

  if (!counting) {
    if (wasDetecting) {
      noDetectionSince = currentMillis;
      wasDetecting = false;
    }

    if (currentMillis - noDetectionSince >= 10000) {
      for (int i = 0; i < NUM_LEDS; i++) {
        strip.setPixelColor(i, strip.Color(0, 0, 255));
      }
      elapsedSessionTime = 0;  // Alleen sessietimer resetten
      melodyPlayed = false;
    }
  } else {
    wasDetecting = true;

    for (int i = 0; i < NUM_LEDS; i++) {
      strip.setPixelColor(i, strip.Color(0, 255, 0));
    }

    if (seconds <= 30) {
      int ledsToYellow = map(seconds, 0, 30, 0, NUM_LEDS);
      for (int i = 0; i < ledsToYellow; i++) {
        strip.setPixelColor(i, strip.Color(255, 255, 0));
      }
    } else if (seconds <= 60) {
      int ledsToRed = map(seconds, 30, 60, 0, NUM_LEDS);
      for (int i = 0; i < NUM_LEDS; i++) {
        strip.setPixelColor(i, strip.Color(255, 255, 0));
      }
      for (int i = 0; i < ledsToRed; i++) {
        strip.setPixelColor(i, strip.Color(255, 0, 0));
      }
    } else if (seconds <= 90) {
      int ledsToPurple = map(seconds, 60, 90, 0, NUM_LEDS);
      for (int i = 0; i < NUM_LEDS; i++) {
        strip.setPixelColor(i, strip.Color(255, 0, 0));
      }
      for (int i = 0; i < ledsToPurple; i++) {
        strip.setPixelColor(i, strip.Color(128, 0, 128));
      }
    }
  }

  strip.show();

  // Check of alle leds dezelfde kleur hebben
  uint32_t firstColor = strip.getPixelColor(0);
  bool allSame = true;
  for (int i = 1; i < NUM_LEDS; i++) {
    if (strip.getPixelColor(i) != firstColor) {
      allSame = false;
      break;
    }
  }

  // Als alle leds dezelfde kleur hebben én het deuntje is nog niet gespeeld
  if (allSame && !melodyPlayed) {
    playingMelody = true;
    currentNote = 0;
    lastNoteTime = millis();
    melodyPlayed = true;  // zorg dat het maar één keer gebeurt
  }
  }
